/************************************************************************
**
** @(#)bgibase.cpp		11/17/94		Chris Ahlstrom
**
** C++
**
** Edit with tab-stops set to 4.
**
**		This module defines the BGIManager class, which implements
** the common functions needed for dealing with the Borland BGI library
** and conversions for drawing objects using the BGIManager
** and ResponseTokenInterior classes.
**
*************************************************************************/

#define BGIBASE_cpp

#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <graphics.h>

#define USE_RESPLIGHT_DATA		// allow inclusion of statics in rbox_afc.h

#include "bios_vid.h"
#include "grafxscr.h"

#include "bgibase.h"			// BGIManager class and ancestors

extern "C"						// avoid name-mangling
{
	#include "math_aux.h"		// simple math like iround()
	#include "textlog.h"		// getScreenModes() [graphics code!]
}


/************************************************************************
** Herc_Alt_driver_far()
**
**   This function is *not* callable... it is here only to allow
** linking in the patched HERC graphics driver.  See the makefile
** (makefile.new).  Note the difference between the two declarations,
** too.
**
**		We must use the far versions, to avoid _TEXT exceeding 64K.
**
*************************************************************************/

#if defined(USE_NEAR_BGI)					// no can do this

extern void _Cdecl Herc_Alt_driver ( void );

#else

extern int far _Cdecl Herc_Alt_driver_far[];

#endif

int	far _Cdecl registerfarbgifont(void far *__font);


/************************************************************************
** BGIManager constructor and destructor.
*************************************************************************/

BGIManager::BGIManager
(
	ScreenCombo combo
) :
	respRows			(25),
	respColumns			(80),
	respPixelX			(0),
	respPixelY			(0),
	graphicsReady		(0),
	monitorCombo		(BGI_USE_MAIN_VGA)
{
	enum graphics_errors errcode = graphicsUp(combo);	// get pixel sizes
	if (errcode == grOk)
		graphicsReady = 1;						// can use graphics now
}


/************************************************************************
** BGIManager destructor
**
**		I'm wondering if it's safe to put the responseOff(1) call
** inside the ResponseInterior destructor instead.  I don't remember
** the order of destructor calls in inheritance.
**
*************************************************************************/

BGIManager::~BGIManager ()
{
	closegraph();
}

/************************************************************************
** graphicsOK()
**
**		For callers, tells them that they can proceed.
**
*************************************************************************/

int
BGIManager::graphicsOK ()
{
	return graphicsReady;
}


/************************************************************************
** graphicsUp()
**
**	Puts the main adapter in graphics mode, assuming the VGA
** driver is part of the executable module (see the UTIL.DOC provided
** by Borland, I believe).
**
**
** "#if defined(USE_NEAR_BGI)"
**
**		We restrict ourselves to the FAR memory model, and have generated
** far versions of the graphics drivers.  See makefile.bcc for more
** notes.
**
** "#if defined(CAN_USE_MORE_MEMORY)"
**
**		Right now, I have an application in which memory is short, and,
** even though I overlay it, I do not wish to support excess drivers
** and character sets.  Therefore, I do not define CAN_USE_MORE_MEMORY,
** and this prevents the use of the normal [unpatched; not "Alt"]
** Hercules driver and the sans-serif font.
**
*************************************************************************/

static int graphDriver;
static int graphMode;

enum graphics_errors
BGIManager::graphicsUp
(
	ScreenCombo combo			// which type of screen to use
)
{

#if defined(USE_NEAR_BGI)					// No can do this; must use far

	void (*driverptr)();		// will point to proper driver code
	void (*fontptr)();			// will point to a stroked BGI font

#else

	void *driverptr;			// will point to proper far driver array
	void *fontptr;				// will point to a far stroked BGI font array

#endif

	int *gd = &graphDriver;			// use DetectGraph (if 0), or driver
	int *gm = &graphMode;			// if not zero, this is graphics mode
	char *dpath = "";				// no location to look for drivers
	enum graphics_errors errcode;	// a return code

	graphDriver	= DETECT;
	graphMode	= 0;
	detectgraph(gd, gm);						// find basic facts
	errcode = (graphics_errors) graphresult();
	if (errcode != grOk)						// no graphics hardware
		return errcode;

	switch (combo)
	{
	case BGI_USE_MAIN_VGA:

		driverptr	= EGAVGA_driver_far;
#if defined(CAN_USE_MORE_MEMORY)
		fontptr		= sansserif_font_far;		// fluff!
#else
		fontptr		= triplex_font_far;
#endif
		graphDriver	= VGA;
		graphMode	= VGALO;					// 640 x 200
		respPixelX	= 640;
		respPixelY	= 200;
		break;

	case BGI_USE_MONO_PATCH:

		driverptr	= Herc_Alt_driver_far;
		fontptr		= triplex_font_far;
		graphDriver	= HERCMONO;
		graphMode	= HERCMONOHI;
		respPixelX	= 720;
		respPixelY	= 348;
		break;

	case BGI_USE_HERC:

#if defined(CAN_USE_MORE_MEMORY)
		driverptr	= Herc_driver_far;
#else
		driverptr	= Herc_Alt_driver_far;
#endif
		fontptr		= triplex_font_far;
		graphDriver	= HERCMONO;
		graphMode	= HERCMONOHI;
		respPixelX	= 720;
		respPixelY	= 348;
		break;

	case BGI_AUTO_DETECT:

		driverptr	= 0;
		fontptr		= 0;
		graphDriver	= DETECT;
		graphMode	= 0;
		Screen *sc = new Screen;					// allocate a struct
		if (sc)
		{
			(void) getScreenModes(sc);				// obtain many parameters
			respRows		= sc->screen_rows;		// number of text rows
			respColumns		= sc->screen_columns;	// number of text columns
			respPixelX		= sc->screen_gmax_x;	// pixel depth
			respPixelY		= sc->screen_gmax_y;	// pixel width
			delete sc;								// get rid of struct
		}
		break;
	}

#if defined(USE_NEAR_BGI)					// no can do this

	if (registerbgidriver(driverptr) < 0)
		errcode = grInvalidDriver;
	else if (registerbgifont(fontptr) < 0)
		errcode = grInvalidFont;

#else

	if (registerfarbgidriver(driverptr) < 0)
		errcode = grInvalidDriver;
	else if (registerfarbgifont(fontptr) < 0)
		errcode = grInvalidFont;

#endif

	if (!errcode)
	{
		initgraph(gd, gm, dpath);					// initialize graphics board
		errcode = (graphics_errors) graphresult();	// check the status
		monitorCombo = combo;
	}

	return errcode;
}


#if defined(__BORLANDC__)
#pragma warn -sig				// { ignore double-to-int warning
#endif

/************************************************************************
** lightAspectConvert()
**
**		Determines the coordinates of a response light relative to
** the response-box window in which it resides, in the same units as
** the response-box window.
**
**		Also copies some other fields from the source (virtual) light.
**
**		Guide to conversions.  The "screen_" values are as per
** "screen.c" and "screen.h" in the FACE or FACE_TV library.
**
** 1. Converting virtual coordinates to characters:
**
**		a. screen_x = (virtual_x / VIRTmax) * screen_columns
**		b. screen_y = (virtual_y / VIRTmax) * screen_rows
**
** 2. Converting virtual coordinates to pixels:
**
**		a. screen_x = (virtual_x / VIRTmax) * screen_gmax_x
**		b. screen_y = (virtual_y / VIRTmax) * screen_gmax_y
**
**		So, in general, if "c" == "coordinate", then
**
**			screen_c = virtual_c * (screen_max_c / VIRTmax)
**
** Now, the "bounds" box (wbox below) gives us some ability to alter
** the overall size of the light-display, as well as the location.
** The units of the box-size are in percent of the screen size.
**
**		Ignoring the offset, including the size of the box changes the
** equation to
**
**			screen_c = virtual_c * (box_c / VIRTmax)
**
*************************************************************************/

void
BGIManager::lightAspectConvert
(
	const Rectangle &wbox,		// window box into which the light goes
	ResponseBoxLight &source,	// pixel-coordinates of the light
	ResponseBoxLight &dest		// dest-coordinate-offsets of the light
)
{
	int va, vb;					// more temporaries
	double maxx;				// virtual-to-video factor (and a temporary)
	double maxy;				// virtual-to-video factor (and a temporary)
	double boxx;				// width of used box
	double boxy;				// height of used box
	int offx;					// offset of x-axis of box
	int offy;					// offset of y-axis of box

	maxx = 0.01 * respPixelX;	// for temporary use only
	maxy = 0.01 * respPixelY;	// for temporary use only

	offx = iround((double)wbox.a.x * maxx);		// convert from % to fraction
	offy = iround((double)wbox.a.y * maxy);		// convert from % to fraction

	boxx = (double) (wbox.b.x-wbox.a.x) * maxx; 
	boxy = (double) (wbox.b.y-wbox.a.y) * maxy;

	dest.fillstyle	= source.fillstyle;			// style of filled figure
	dest.color		= source.color;				// color of response light

	if (boxx >= 1.0 && boxy >= 1.0)				// good bounds given
	{
		maxx = (boxx / VIRTmax);
		maxy = (boxy / VIRTmax);
	}
	else										// use whole screen
	{
		maxx = respPixelX / VIRTmax;
		maxy = respPixelY / VIRTmax;
	}

	va = iround(source.x0 * maxx);
	vb = iround(source.x1 * maxx);
	if ((vb-va) <= 0)							// keep it > 0
		vb = va + 1;
	dest.x0 = va + offx;
	dest.x1 = vb + offx;

	va = iround(source.y0 * maxy);
	vb = iround(source.y1 * maxy);
	if ((vb-va) <= 0)							// keep it > 0
		vb = va + 1;
	dest.y0 = va + offy;
	dest.y1 = vb + offy;
}


/************************************************************************
** textAspectConvert()
**
**		Once the above works, fix this to suit.
**
*************************************************************************/

void
BGIManager::textAspectConvert
(
	const Rectangle &wbox,		// window box into which the light goes
	ResponseBoxText &source,	// pixel-coordinates of the light
	ResponseBoxText &dest		// dest-coordinate-offsets of the light
)
{
	double maxx;				// virtual-to-video factor (and a temporary)
	double maxy;				// virtual-to-video factor (and a temporary)
	double boxx;				// width of used box
	double boxy;				// height of used box
	int offx;					// offset of x-axis of box
	int offy;					// offset of y-axis of box

	maxx = 0.01 * respPixelX;	// for temporary use only
	maxy = 0.01 * respPixelY;	// for temporary use only

	offx = iround((double)wbox.a.x * maxx);		// convert from % to fraction
	offy = iround((double)wbox.a.y * maxy);		// convert from % to fraction

	boxx = (double) (wbox.b.x-wbox.a.x) * maxx; 
	boxy = (double) (wbox.b.y-wbox.a.y) * maxy;

	// dest.fillstyle	= source.fillstyle;			// style of filled figure
	// dest.color		= source.color;				// color of response light

	dest.font			= source.font;				// font of the text
	dest.direction		= source.direction;			// direction of text
	dest.charsize		= source.charsize;			// size of text
	dest.textstring		= source.textstring;		// the actual text

	if (boxx >= 1.0 && boxy >= 1.0)				// good bounds given
	{
		maxx = (boxx / VIRTmax);
		maxy = (boxy / VIRTmax);
	}
	else										// use whole screen
	{
		maxx = respPixelX / VIRTmax;
		maxy = respPixelY / VIRTmax;
	}
	dest.x0 = iround(source.x0 * maxx) + offx;
	dest.y0 = iround(source.y0 * maxy) + offy;
}


/************************************************************************
** screenX()
** screenY()
**
**		Quick-and-dirty routines to convert Hercules coordinates
** to the coordinates of the current screen (either Hercules or VGA).
** Helps in retrofitting some old code, in which the coordinates were
** hard-wired to Hercules values.
**
*************************************************************************/

int
BGIManager::screenX
(
	int hercx
)
{
	return int(double(hercx) * double(respPixelX) / 720.0);
}


int
BGIManager::screenY
(
	int hercy
)
{
	return int(double(hercy) * double(respPixelY) / 348.0);
}


/************************************************************************
** mouseX()
** mouseY()
**
**		Quick-and-dirty routines to convert VGA 640 x 200 coordinates
** to Hercules coordinates.  Helps in retrofitting some new code.  We
** do not know how to get the mouse driver to support a cursor on the
** alternate monitor, so these routines help the normal mouse driver
** support drawing a cursor in the proper place on either the VGA or
** Hercules screen, given VGA coordinates as input.
**
*************************************************************************/

int
BGIManager::mouseX
(
	int vgax
)
{
	return int(double(vgax) * double(respPixelX) / 640.0);
}


int
BGIManager::mouseY
(
	int vgay
)
{
	return int(double(vgay) * double(respPixelY) / 200.0);
}


/************************************************************************
** hercX()
** hercY()
**
**		Quick-and-dirty routines to convert VGA 640 x 200 coordinates
** to Hercules coordinates.
**
**		This is similar to the mouseX() and mouseY() routines, except
** that the conversion is the same no matter what the current monitor
** is.
**
**		Consider these routines as mappers from a "mouse coordinate
** system" to a "virtual coordinate system", where the latter happens
** to be the Hercules coordinate system.
**
*************************************************************************/

int
BGIManager::hercX
(
	int mousex
)
{
	return int(double(mousex) * 720.0 / 640.0);
}


int
BGIManager::hercY
(
	int mousey
)
{
	return int(double(mousey) * 348.0 / 200.0);
}


#if defined(__BORLANDC__)
#pragma warn .sig				// } put warning back to original state
#endif

