/************************************************************************
**
** @(#)chkflt.cpp		08/05/94		Chris Ahlstrom
**
** Edit with tab-stops at 4.
**
** ------------------------
** 73340.26@compuserve.com
** ------------------------
**
**		Manages a CheckListFloat dialog box.  This is the CheckListBase
** dialog class with support for converting between strings and floating
** point numbers.  See chklist.cpp/h.
**
*************************************************************************/

#define CHKFLT_cpp

#include <math.h>				// atof()

#include "chkflt.h"			// external declarations


/************************************************************************
** CheckListFloat()
**
**		The constructor assumes that the caller has set up the following
** items:
**
**		1.	An array of 'maxitem' integers to reflect the usage-status of
**			'maxitem' items of information.
**
**		2.	A corresponding array of unspecified structures that contains
**			all information for each item.  For example, it might
**			consist of an array of 'maxitem' floating-point numbers.
**
**			The default implementation assumes the items consist of an
**			array of string pointers.
**
*************************************************************************/

CheckListFloat::CheckListFloat
(
	TDeskTop *desk,				// TurboVision desktop
	int checkoptions,			// word containing desired check-list options
	const char *maintitle,		// string to use for main title
	const char *itemtitle,		// string to use for item title
	int *status,				// array word encoding status of each item
	float *items,				// pointer to array of items
	int maxitem,				// count of number of items
	int initpage,				// current page in list [default = 1]
	char *optionlabel			// one option label for now
) :
	CheckListBase
	(
		desk, checkoptions, maintitle, itemtitle,
		(CheckListBits *) status, (void *) items,
		maxitem, initpage, optionlabel
	)
{
}


CheckListFloat::~CheckListFloat ()
{
}


/************************************************************************
** setItemString()
**
**		Gets the item string *from* the caller.
**
**		It gets the item'th floating point number in an array of
** floating point numbers, and converts it to a string.
**
**		The overriding version should also be prepared to convert
** useless entries to a string such as "<inactive>".  The routine
** here does not know of such useless entries; the only way to tell
** the dialog box that the number is inactive is to lower the
** CHKLIST_ACTIVE bit in the corresponding status word.
**
**		[However, as seen in getItemString(), the user can make a field
** inactive henceforth, by entering a non-number or empty value.]
**
*************************************************************************/

#define CHK_FLT_FORMAT	" %13.8g"					// display format

void
CheckListFloat::setItemString
(
	int item
)
{
	float *floatarray = (float *) callerItems;		// get pointer to 1st
	int status = callerStatus[firstItem+item];		// caller has set status

	if (status & CHKLIST_ACTIVE)
	{
		(void) sprintf
		(
			statusArea.itemString[item],			// destination string
			CHK_FLT_FORMAT,							// display format
			*(floatarray+firstItem+item)			// the number
		);
	}
	else
		(void) strcpy(statusArea.itemString[item], "<inactive>");
}


/************************************************************************
** getItemString()
**
**		Gets the item string and reformats it to put it back *to* the
** caller.  This routine also puts back the status field, too.
**
**		This will probably be overridden a lot.  The default here just
** copies the string back to the right string as pointed to by a
** pointer in an array of pointers.  You can imagine that someone might
** want to instead convert the string to a floating point number.
**
**		The overriding version should also be prepared to convert
** a string such as "<inactive>" or "-1" to a useless entry.  Here,
** any string that is not a good number (after skipping the initial
** white space) renders the item inactive.
**
*************************************************************************/

static inline int
isnumeric
(
	char digi
)
{
	return
	(
		(digi >= '0' && digi <= '9') ||
		(digi == '.') || (digi == '-') || (digi == '+') ||
		(digi == 'e') || (digi == 'E')
	);
}

void
CheckListFloat::getItemString
(
	int item
)
{
	float *floatarray = (float *) callerItems;		// pointer to 1st float
	char *text = &statusArea.itemString[item][0];	// pointer to item
	int countdown = CHK_ITEM_CHARS;					// maximum chars in item

	while (*text == ' ' && *text != '\0')			// skip initial white space
	{
		if (--countdown == 0)
			break;
		else
			text++;
	}
	if ((countdown == 0) || !isnumeric(*text))
	{												// don't have good number
		*(floatarray+firstItem+item) = 0.0f;
		callerStatus[firstItem+item] = CHKLIST_UNUSED;
	}
	else											// have good number-string
	{
		*(floatarray+firstItem+item) = (float) atof(text);
		callerStatus[firstItem+item] = statusArea.itemStatus[item];
	}
}

