/************************************************************************
**
** @(#)chklbits.h		08/05/94		Chris Ahlstrom
**
**		Manages the CheckListDialog for the DOS version.  Similar in
** some ways to the x_calpk.cpp/h/men modules.  See them for further
** commentary.
**
*************************************************************************/

#ifndef CHKLBITS_h									// { CHKLBITS_h
#define CHKLBITS_h


/************************************************************************
** CheckListFunction
**
**		Through this enumeration, the use can give the checklist and
** the caller instructions as to how the checklist is to be used.
**
**		Due to room in the dialog [chklist.men], we can only have 14
** options, not the full 16.
**
*************************************************************************/

#define CHK_OPTION_1_INDEX	 8			// first available option
#define CHK_OPTION_LENGTH	20			// characters we can write to

typedef enum
{
	CHKFUNC_NONE			= 0x0000,	// no bits set
	CHKFUNC_USE_THIS		= 0x0001,	// use checklist in other commands
	CHKFUNC_RANDOMIZE		= 0x0002,	// randomize checklist during setup
	CHKFUNC_DISPLAY_PAGE	= 0x0004,	// display checklist before each use
	CHKFUNC_SINGLE_RUNS		= 0x0008,	// allow other work between loops
	CHKFUNC_SAVE			= 0x0010,	// save whole config after setup
	CHKFUNC_PARAMETER_PROMPT= 0x0020,	// prompt for parameters
	CHKFUNC_EXPANSION_1		= 0x0040,	// reserved for expansion
	CHKFUNC_EXPANSION_2		= 0x0080,	// reserved for expansion
	CHKFUNC_OPTION_1		= 0x0100,	// reserved for expansion
	CHKFUNC_OPTION_2		= 0x0200,	// reserved for expansion
	CHKFUNC_OPTION_3		= 0x0400,	// reserved for expansion
	CHKFUNC_OPTION_4		= 0x0800,	// reserved for expansion
	CHKFUNC_OPTION_5		= 0x1000,	// reserved for expansion
	CHKFUNC_DEBUG_OPTION	= 0x2000,	// just do setup [aid to debugging]

	CHKFUNC_ALL				= 0x3FFF	// max is 14 options

} CheckListFunction;


/************************************************************************
** CheckListBits
**
**		While the Turbo-Vision data transfer area check-box lets us
** represent just one kind of status [whether or not the item is checked],
** we also want to be able to encode other statuses to help in creating
** the dialog.
**
**		For example, we want to be able to indicate if an item is even
** active [selectable].
**
*************************************************************************/

typedef enum
{
	CHKLIST_UNUSED			= 0x00,		// no bits set, zeroed
	CHKLIST_SELECTED		= 0x01,		// item should have a check-mark
	CHKLIST_ACTIVE			= 0x02,		// item should not be ghosted
	CHKLIST_UNUSABLE		= 0x04,		// item can never be used

	CHKLIST_ALL_BITS		= 0x07		// just for clerical usage

} CheckListBits;


#endif												// } CHKLBITS_h
