/************************************************************************
**
** @(#)date.c			08/05/94		Chris Ahlstrom
**
**		Contains date and time functions. Part of the FACE library.
** Allows easy display of the current time and date in the formats
** 09-03-95
**
*************************************************************************/

#include "date.h"


/************************************************************************
** dos_date (formerly called date)
**
**		Gets the date from DOS, formats it, and puts it into the string
** provided.
**
*************************************************************************/

static struct date date_struc;

void
dos_date
(
	char *date_str
)
{
	getdate(&date_struc);
	date_struc.da_year -= 1900;
	if (date_struc.da_year >= 100)
		date_struc.da_year -= 100;

	(void) sprintf
	(
		date_str, "%2d/%2d/%2d\0",
		date_struc.da_mon,
		date_struc.da_day,
		date_struc.da_year
	);
}


/************************************************************************
** dos_time (formerly called time)
**
**		Gets the time from DOS, formats it, and puts it into the string
** provided.
**
*************************************************************************/

static struct time time_struc;

void
dos_time
(
	char *time_str
)
{
	gettime(&time_struc);

	(void) sprintf
	(
		time_str, "%2d:%2d:%2d\0",
		time_struc.ti_hour,
		time_struc.ti_min,
		time_struc.ti_sec
	);
}



/************************************************************************
** calculate_age()
**
**		Attempts to get the age from the Birth and Date strings
** provided.
**
**		This algorithm depends upon each string being properly formatted.
** I've used one template for each in case the user enters the birthdate
** in a format a little different from that in the Date field [which
** is filled in by the computer.]
**
**		Pretty shaky way to do things!  It would be better to convert
** the string to an internal format, and subtract, then round to the
** nearest year, I think.  Later, dudes.
**
*************************************************************************/

static char *dob_template = "%d/%d/%d";
static char *doy_template = "%d/%d/%d";

int
calculate_age
(
	char *dob,			/* date-of-birth string			*/
	char *doy			/* today's-date string			*/
)
{
	int by, bm, bd;					/* birth year, month, day			*/
	int cy, cm, cd;					/* current year, month, day			*/
	int age;						/* the return value	in years		*/
	char dob_sep, doy_sep;			/* separator as existing			*/
 
	dob_sep = dob[2];				/* examine the input separator		*/
	dob_template[2] =
	dob_template[5] = dob_sep;		/* put it into template string		*/
	sscanf(dob, dob_template, &bm, &bd, &by);
 
	doy_sep = doy[2];				/* examine the input separator		*/
	doy_template[2] =
	doy_template[5] = doy_sep;		/* put it into template string		*/
	sscanf(doy, doy_template, &cm, &cd, &cy);

	age = cy - by;
	if ((cm < bm) || ((cm == bm) && (cd < bd)))
		age--;

	return age;
}

