/************************************************************************
**
** @(#)face_box.h		12/08/94		Chris Ahlstrom
**
**	Edit with tab-stops = 4.
**
**	C/C++ compatible. Consists of at least three sections:
**
**		1.	The DAC (D/A converter) Parameters dialog box, stolen from
**			the mod_dac modules of MODAPP, used here purely as a test
**			bed for dialog box control. This is a good test of the
**			general data scheme.
**
**		2.	We also provide a section showing how to use the MappedField
**			elements of tv_menu.h. Here, the setup is a little more
**			complex and indirect.
**
**		3.	The experimental parameters dialog box copped from the 
**			mod_exp modules of MODAPP.  Just includes a few more type
**			of data than item 1.
**
**		4.	CheckListFloat test code.
**
**		This header file describes sample data-structures that
** we can use with the dialog box code. It illustrates the hard-wired
** DataTypes, byte-mapped data-types, and Extended data-types.
**
*************************************************************************/

#if !defined(FACE_BOX_h)							/* { FACE_BOX_h		*/
#define FACE_BOX_h


/************************************************************************
** FAR
**
**		We got a problem with DGROUP being more than 64K, what with
** FACE_TV, XBTOOLS, ADAPTIVE, and normal application static variables.
** Therefore, some of the statics here and in those libraries have
** been rendered "far". To make them not far, simply place
** "#define FAR" before the #include.
**
**		Note that another option would be to include the compiler flag
** "-Ff=nnnn", which would set the threshold for making statics far
** to nnnn. However, it's not obvious what a good threshold would be,
** and perhaps we'd end up with an error about too many segments.
**
*************************************************************************/

#include "far.h"			/* define FAR								*/

#include "rbuttons.h"		/* FACE_TV "button devices"					*/
#include "tvyesno.h"		/* FACE_TV dialog Yes/No types				*/
#include "tv_menu.h"		/* FACE_TV dialog menu types				*/


/************************************************************************
** Some convenient data types. All this stuff was copped from the
** mod_dac and dt2801a modules of MODAPP, and stripped down just
** for creating a very simple testbed for the FACE_TV code.
*************************************************************************/

#define AMPL_ONE 10000					/* our version of "1"			*/
#define DAC_RATE 10000					/* general default DAC rate		*/
#define DACNSIZE		 15+1			/* size of the name string		*/
#define NOTESIZE		 120+1			/* a large string				*/
#define MAX_DAC_VOLTAGE	 9.9951			/* the realistic board maximum	*/
#define MAX_DAC_OUTPUT	4095			/* the board can put out this	*/
#define MIN_DAC_VOLTAGE	 -9.9951		/* the realistic board minimum	*/
#define MIN_DAC_OUTPUT	 1				/* ditto						*/

typedef int DACSignal;
typedef char DacnString[DACNSIZE];		/* test of String datatype		*/
typedef char CharString[DACNSIZE-1];	/* test of ByteString datatype	*/
typedef char NoteString[NOTESIZE];		/* test of SCROLL_END feature	*/

typedef enum
{
	DT2801_DEVICE = 0,
	NUCLEUS_DEVICE,
	LSPSS_DEVICE,
	TEST_DEVICE,						/* calls test routines			*/
	MISSING_DEVICE

} DeviceType;


/************************************************************************
** DacByte
**
**		We define the different types of DAC data (apart from the
** normal types such as Integer and Float).  We use them in the
** sample menu shown in FACE_BOX.MEN. Used for testing
** ExtendedDescriptors.
**
** MidByte
**
**		Similar in concept, but used for testing MappedFieldDescriptors,
** which are a little more complex than ExtendedDescriptors.
**
*************************************************************************/

typedef enum
{
	DAC_FLOAT			= 0,
	DAC_INTEGER,
	DAC_UNSIGNED_BYTE

} DacByte;

typedef enum
{
	MID_BYTE			= 0,
	MID_SBYTE,
	MID_STRING,
	MID_PART

} MidByte;


/************************************************************************
** DACParameters (dialog box)		[stolen and modified from mod_dac]
**
**		Just a good, complex test structure.
**
*************************************************************************/

typedef struct
{
	DeviceType device;			/* device-type radio button				*/
	DacnString name;			/* nickname for the device?				*/
	CharString max;				/* bogus field for dialogue testing		*/
	double outputRate;			/* DAC sampling rate					*/
	int nPoints;				/* max number of points in the buffer	*/
	YesNoType testCode;			/* bogus test code for showing Yes/No's	*/

#ifdef USE_DACRATE_CODE_INSTEAD	/* I have a more important test now		*/
	unsigned dacRate;			/* a mappable field						*/
#else
	NoteString note;			/* string that is scrollable			*/
#endif

	int pt_start;				/* start point (usually 0)				*/
	double us_start;			/* start time (micro-sec, usually 0)	*/
	int pt_end;					/* end point (sometimes nPoints-1)		*/
	double us_end;				/* end time (in microseconds)			*/
	int dacMin;					/* lowest digital value (usually 0)		*/
	double mVmin;				/* lowest signal level in mV			*/
	int dacMax;					/* highest digital level				*/
	double mVmax;				/* highest signal level in mV			*/

	double calibFreq;			/* calibration frequency				*/
	double calibVolts;			/* calibration voltage					*/

} DACParameters;


/************************************************************************
** MIDParameters (dialog box)
**
**		Just a good, complex test structure if MappedFieldDescriptors.
**
*************************************************************************/

typedef struct
{
	char name[10];
	unsigned char tempo;
	signed char pan;
	char part;

} MIDParameters;


/************************************************************************
** Various EXP macros
**
**		They had meaning in the original MODAPP code [mod_exp.men],
** but here they are included just to satisfy the compiler.
**
*************************************************************************/

#define NUMB_OUT				 4
#define MAX_REVERSALS			 48
#define MINUS_INFINITY_DB		-99.0


/************************************************************************
** Various sample typedefs
*************************************************************************/

typedef int DataWord;
typedef DataWord ReverseArray[MAX_REVERSALS];

typedef enum
{
	MAIN_SCREEN					= 0,
	MONO_SCREEN,
	ALT_MONO_SCREEN

} ScreenSetup;

typedef enum
{
	LAYOUT_AFC2					= 0,
	LAYOUT_TOKENS

} ExperimentLayout;

typedef enum
{
	STEP_LINEAR					= 0,
	STEP_NONLINEAR

} Linearity;


/************************************************************************
** ResponseCombo
**
**		The type of device-combination the subject will be viewing
** and responding with.
**
**		At MUSC, we intend the following meanings:
**
** LAB_MOUSE_SCREEN	Alternate mouse and monochrome screen
**						 in the subject booth.
**
** MAIN_MOUSE_SCREEN	Main mouse and color screen in the
**						 experimentor's antechamber.
**
** KEYBOARD_SCREEN		Keyboard and color screen (in
**						 labs with only one mouse.)
**
** DEFAULT_EQUIPMENT	Use BGI autodetection and the current mouse.
**
*************************************************************************/

typedef enum
{
	LAB_MOUSE_SCREEN			= 0,
	MAIN_MOUSE_SCREEN,
	KEYBOARD_SCREEN,
	DEFAULT_EQUIPMENT

} ResponseCombo;


/************************************************************************
** ExpParameters (dialog box)		[mod_exp]
**
**		An interface between the experimental-parameter dialog box
** and the AFCTask class.
**
**		Note the extra padding, which gives us some minor room to
** expand and still be able to read older configuration files.
** When the padding is used up, it's time to byte-edit your old
** configuration files, or recreate them!
**
**		Note that we actually use the scrType and responseDevice
** fields elsewhere, in the response-box test commands.
**
*************************************************************************/

typedef struct
{
	ScreenSetup scrType;				/* type of screen radiobox		*/
	ExperimentLayout expCode;			/* type of comparisons made		*/
	ResponseType responseDevice;		/* device subject can use		*/
	int numbCorrect;					/* number correct to change		*/
	int afcIntervals;					/* number of intervals (2 only)	*/
	int numbRevQuit;					/* number of 2-AFC reversals	*/
	int numbOut;						/* number to throw out at first	*/
	long afcSpeed;						/* long code for testing Longs	*/
	double afcStep;						/* control over step-size		*/
	double afcHalfStep;					/* control over half-step size	*/
	double afcFloor;					/* control over the floor value	*/
	double afcCeiling;					/* control over ceiling value	*/
	YesNoCheck valueTrack;				/* controls display of values	*/
	YesNoCheck stimPeek;				/* controls display of interval	*/
	YesNoCheck noisePeek;				/* controls display of noise	*/
	Linearity stimLinearity;			/* linearity of stimulus levels	*/
	Linearity stepLinearity;			/* linearity of steps			*/

	char padding[10];					/* room for expansion			*/

} ExpParameters;


/************************************************************************
** FreqCheckList
*************************************************************************/

#include "chklbits.h"	/* CheckListFunction and CheckListBits types	*/

#define FREQS_SUPPORTED			20

typedef struct
{
		int freqCount;
		int status[FREQS_SUPPORTED];
		float frequency[FREQS_SUPPORTED];
		int options;

} FreqCheckList;

FreqCheckList FAR frequencyList =
{
	FREQS_SUPPORTED,					// int freqCount;
	{									// int status[FREQS_SUPPORTED];
		0x03,							// active & selected
		0x02,							// active only
		0x01,							// selected only
		0x00							// the rest are unused
	},
	{									// float frequency[FREQS_SUPPORTED];
		200.0f,
		250.0f,
		315.0f,
		400.0f,
		500.0f,
		630.0f,
		800.0f,
		1000.0f,
		1250.0f,
		1600.0f,
		2000.0f,
		2500.0f,
		3150.0f,
		4000.0f,
		5000.0f,
		6300.0f,
		0.0f,
		0.0f,
		0.0f,
		0.0f
	},									// int options;
	CHKFUNC_USE_THIS | CHKFUNC_RANDOMIZE |
	CHKFUNC_DISPLAY_PAGE | CHKFUNC_SINGLE_RUNS
};


/************************************************************************
** Response Token test data
**
**		The screen is internally divided into a 7 x 5 grid. The box
** numbers are thus layed out as follows on the screen:
**
**		 1		 2		 3		 4		 5		 6		 7
**		 8		 9		10		11		12		13		14
**		15		16		17		18		19		20		21
**		22		23		24		25		26		27		28
**		29		30		31		32		33		34		35
**
** When box numbers are provided, some box numbers might not be
** used; no token will appear in that slot.
**
** Our test data are layed out so that box numbers
**
**		1, 2, 3, 4, 5, 6,			// slot 7 unused
**		8, 9, 10, 11, 12, 14, 		// slot 13 unused
**		15, 16, 17, 18, 19, 21, 	// slot 20 unused
**		30, 31, 32, 33				// slot 22-29 & 34-35 unused
**
** are mapped onto the tokens
**
**		B, CH, D, Dg, Dz, F, 
**		G, K, L, M, N, P,
**		R, S, SH, T, Th, Thz, 
**		V, W, Y, Z
**
*************************************************************************/

#include "toklayo.h"

TokenLayout faceTokens[] =
{
	{	 1,	"B"		},
	{	 2,	"CH"	},
	{	 3,	"D"		},
	{	 4,	"Dg"	},
	{	 5,	"Dz"	},
	{	 6,	"F"		},
	{	 8,	"G"		},
	{	 9,	"K"		},
	{	10,	"L"		},
	{	11,	"M"		},
	{	12,	"N"		},
	{	14,	"P"		},
	{	15,	"R"		},
	{	16,	"S"		},
	{	17,	"SH"	},
	{	18,	"T"		},
	{	19,	"Th"	},
	{	21,	"Thz"	},
	{	30,	"V"		},
	{	31,	"W"		},
	{	32,	"Y"		},
	{	33,	"Z"		}
};
const int faceTokenCount = (sizeof(faceTokens) / sizeof(TokenLayout));

#endif												/* } FACE_BOX_h		*/
