/************************************************************************
**
** @(#)face_tv.cpp		11/27/94		Chris Ahlstrom
**
** -------------------------
**  73340.26@compuserve.com
** -------------------------
**
** This main program serves as a fairly simple test-bed and demo program
** for the FACE_TV library.
**
**		To build it, use the makefile provided.
**
** v. 1.46	Added a way to handle a grid of items to choose on the screen.
**			This involves a lot of new code in tmouse, rbox_inb, rbox_btk,
**			rbox_dtk, and perhaps more.
**
** v. 1.45	Added dumping of Program Log window to a file.
**
** v. 1.44	Added a way to handle 1 to 3 intervals of lights.
**
** v. 1.43	Added CheckListFloatPair code and ability to switch
**			to alternate mouse in msrbutt.cpp.
**
** v. 1.42	Added CheckListFloat/Base code and test support for it.
**			Handles a very convenient paged-checklist, with options,
**			and a shaky-but-useful interface to application code.
**
** v. 1.41	Added ResponseBGIDevice code, to get ready to
**			use the BGI graphics system, first on the main monitor,
**			then on an alternate monochrome monitor.
**
** v. 1.40	Added ResponseBoxDevice code for more general handling
**			of response-device. Left the ResponseDevice code
**			intact here and in the library. Anyway, the new code
**			has big problems, and I had to get to the work in
**			v. 1.41.
**
** v. 1.31	Added ResponseDevice code to support interactive
**			between the user and the program in experiments.
**			It includes a "response test" command that just
**			verifies that the button and screen code works
**			properly.
**
** v. 1.30	Some old code is still in here... I've incorporated
**			code to test a new way of handling configuration
**			files. It is implemented by creating a
**			SimpleConfiguration object.
**
**			Also involved is a better way of handling error
**			messages (the class provides a pointer to the
**			error message) which doesn't fit into the
**			errorHandler() scheme, alas.
**
*************************************************************************/


/************************************************************************
**
** MAIN_cpp			used for defining variables
** FACE_TV_cpp		used for definitions peculiar to FACE_TV
** PROGRAM_VERSION	appears in banners
** FACE_DOS_PROMPT	prompt that appears on DOS command line
** FACE_WILDCARD	generic extension for FACE_TV-related files
**
*************************************************************************/

#define MAIN_cpp					// only one of these per program
#define FACE_TV_cpp					// only one of these per FACE_TV
#define PROGRAM_VERSION	" 1.46"		// Turbo Vision is 2nd generation

#define FACE_DOS_PROMPT	"PROMPT='EXIT' returns to FACE$_$P$G"
#define FACE_WILDCARD	"*.FAC"

#include <stdlib.h>			// for exit(), random()
#include <conio.h>			// for video modes
#include <iostream.h>
#include <fstream.h>		// for ifstream
#include <stdio.h> 			// for puts() etc
#include <string.h>			// for strlen etc
#include <ctype.h>

#define USE_GEN_ERRORS		// include message buffer in announce.h
#define USE_YES_NO_BUTTONS	// include YesNo indirectly in announce.h
#define USE_TURBO_VISION	// use TV version of screenmsg()
#include "announce.h"		// error message and general message code
#include "boxtools.h"		// tools for global annunciator boxes
#include "chkflt.h"			// CheckListFloat class for fancy dialog
#include "face_tv.h"		// a good description of the main program
#include "face_tv.men"		// all the menu entries
#include "face_box.men"		// a sample setup for the dialog box

#if defined(TESTHEAP)		// { TESTHEAP
#include "heapview.h"		// handy viewer for debugging heap problems
#endif						// } TESTHEAP

#include "tinpmous.h"		// controlling numbers with the mouse
#include "tv_confg.h"		// SimpleConfiguration class
#include "respwind.h"		// ResponseDevice class


/************************************************************************
** Static bar pointers for the main program
*************************************************************************/

const NestBar *MenuBarApp::menuBar = &FACEMenuBar;
const StatusBar *MenuBarApp::statusBar = &FACEStatusBar;


/************************************************************************
** TFaceApp constructor
*************************************************************************/

TFaceApp::TFaceApp()
 :
	TProgInit						// initialize the desktop
	(
		TFaceApp::initStatusLine,
		TFaceApp::initMenuBar,
		TFaceApp::initDeskTop
	),
	MenuBarApp			(),						// initialize menubar
	TBox										// init dialog
	(
		deskTop, dacByteList, midByteList
	),
	DosShell			(FACE_DOS_PROMPT),		// set up DOS prompt
	appError			(ERR_NONE),				// start with no errors
	showBox				((TDialog *) NULL),		// control box
	equipmentConfig		(0),
	modelessDacDialog	(0),
	modelessDlgPtr		(0)
{
	msgArea = new UserMessages(deskTop, genErrors, genErrorsLength);

	/********************************************************************
	** Should check the following for success (I do in FACE_WIN, but
	** not here, tsk tsk.)
	*********************************************************************/

	equipmentConfig = new SimpleConfiguration
	(
		deskTop, "default.ftv", &equipmentList[0] // default extension "FTV"
	);

	/********************************************************************
	** Let's give a simple test to a useful feature for converting
	** old code to run with Turbo Vision, by letting the user replace
	** puts() [or printf(), if the caller formats the string using
	** sprintf()] calls with calls to screenmsg().  Note we create the
	** log window with 4096 bytes instead of the default 64000.
	*********************************************************************/

	#define SCREEN_LENGTH		4096
	#define SCREEN_FORMAT		"Test Number #%3d\n"
	#define SCREEN_FMT_LEN		17

	testsWindow = (TWindowScreen *) 0;
	if (testsWindow == (TWindowScreen *) 0)
	{
		testsWindow = startWindowScreen(deskTop, 4096U);
		(void) screenmsg
		(
			1, testsWindow, "FACE_TV messages system is running\n"
		);														// 35 bytes
		(void) screenmsg
		(
			1, testsWindow,
			"Can fill log to check\" Dump Log\" command.\n"
				"Log now has 80 bytes.\n\n"
		);														// 45 bytes

		//
		// This test would normally write 240 lines, but since 80 are already
		// written, this should overflow at 236 of them.  It actually
		// overflows at 213, if you enable this code.  So the application
		// must issue "clear" calls periodically to guarantee room
		// in the window.  This seems to be due to a bug in Borland's
		// TTerminal code.
		//
		// char formatstr[SCREEN_FMT_LEN+8];
		//
		// for (int ti = 0; ti < (4096/SCREEN_FMT_LEN); ti++)
		// {
		//		sprintf(formatstr, SCREEN_FORMAT, ti);
		//		(void) screenmsg(1, testsWindow, formatstr);
		// }
		//
	}

#if defined(TESTHEAP)							// { TESTHEAP

	TRect r = getExtent();						// create the heap view
	r.a.x = r.b.x - 13;
	r.a.y = r.b.y - 1;
	heap = new THeapView(r);
	insert(heap);

#endif											// } TESTHEAP

	initTFaceApp();								// set-up initial status
}


/************************************************************************
** TFaceApp destructor
**
**		We don't need to delete any objects that have been inserted
** into the deskTop... the deskTop takes care of calling destroy()
** for any items still inserted into it (such as testsWindow).
**
*************************************************************************/

TFaceApp::~TFaceApp ()
{

#if defined(TESTHEAP)							// { TESTHEAP

	delete heap;

#endif											// } TESTHEAP

	if (modelessDlgPtr)
		modelessDacDialog->undoDialog(modelessDlgPtr);
	if (modelessDacDialog)
		delete modelessDacDialog;
	if (equipmentConfig)
		delete equipmentConfig;
	if (msgArea)
		delete msgArea;
}


/************************************************************************
** TFaceApp::initTFaceApp ()
**
**		Very simple now, but might read configuration files later.
** C4350 is another option.  I don't UNDERSTAND how these work.
** Can't get 50-line mode to come up, and the mouse can move way
** below the screen if DOS was in 50-line mode before you started
** the FACE_TV program!!!!
**
**		For now, I like the behavior of matching the DOS screen mode
** better... if the mode is 50-line, use that, otherwise use the 25-line
** mode. So I've commented out the setScreenMode call.
**
**		Maybe need a better way to handle this bloody issue.
**
**		By the way, CONIO.H has more video mode integers than
** the smxxxx constants in Turbo Vision's SYSTEM.H. And don't bother
** checking... CONIO's constants match (in value) those in SYSTEM.
**
*************************************************************************/

void
TFaceApp::initTFaceApp ()
{
	textMode	= C80;							// C80 is in conio.h

	// textMode		= C80 + C4350;				// C80 is in conio.h
	// setScreenMode(textMode);					// fix the screen

	modelessDacDialog = new DialogBox
	(
		&dacMenu,
		(GeneralDialogData) &dacDialog,
		sizeof(DACParameters),
		deskTop
	);
}


/************************************************************************
** TFaceApp handleEvent ()
**
**		Note that the old way of reading and writing configuration
** information is superceded.  Note also that we could add a third
** configuration command denoted "Save As", copy the calls for the
** Write command to it, and remove the parameter [FORCE_NEW_NAME]
** (or change it to PREVIOUS_NAME_UNLESS_INACTIVE, an ugly alternative),
** to make it more like the standard Save (which normally uses
** the default name, unless none has been given yet).
**
**		See the TV_CONFG modules for more information.
**
*************************************************************************/

void
TFaceApp::handleEvent
(
	TEvent& event
)
{
	ErrorCode he_error = ERR_NONE;
	CheckListFloat *clf = 0;					// only needed by one case

	TApplication::handleEvent(event);			// handle the TV stuff

	if (event.what == evCommand)
	{
		switch (event.message.command)
		{
		case cmConfigWriteAs:

			if (equipmentConfig)
				(void) equipmentConfig->write(FORCE_NEW_NAME);	// get new name
			break;

		case cmConfigWrite:

			if (equipmentConfig)
				(void) equipmentConfig->write();
			break;

		case cmConfigRead:

			if (equipmentConfig)
				(void) equipmentConfig->read(FORCE_NEW_NAME);
			break;

		case cmDumpLog:

			if (testsWindow)
			{
				testsWindow->write();
				(void) screenmsg
				(
						1, testsWindow, "FACE_TV messages system cleared.\n\n"
				);
			}
			break;

		case cmKillLog:

			if (testsWindow)
				delete testsWindow;
			break;

		case cmDOS_Cmd:							// DOS shell

				TApplication::suspend();		// release mouse
			shell();							// shell out to DOS
				TApplication::resume();			// get mouse back
			redraw();							// redraw application screen
			break;

		case cmInfo:

			msgArea->bannerHandler(FACE_TV_BANNER);
			break;

		case cmDacDialog:

			(void) doDialog(&dacMenu, &dacDialog);
			break;

		case cmMidDialog:

			(void) mappedDialog(&midMenu, &midDialog);
			break;

		case cmFreqDialog:

			clf = new CheckListFloat
			(
				deskTop,						// TurboVision desktop
				frequencyList.options,			// desired check-list options
				"Signal Frequencies",			// main title string
				"Used Frequencies (Hz)",		// item title string
				frequencyList.status,			// array encoding statuses
				frequencyList.frequency,		// pointer to array of items
				frequencyList.freqCount			// count of number of items
			);
			if (clf)
			{
				clf->run();
				frequencyList.options = clf->getCheckOptions();
				delete clf;
			}
			else
				he_error = ERR_OBJECT_NOT_CREATED;
			break;

		case cmExpDialog:

			(void) doDialog(&expMenu, &expDialog);
			break;

		case cmDelayDialog:

			(void) doDialog(&delayWindow, &delayData);
			break;

		case cmModalDialog:				// execute new-style modal dialog

			//
			// Not sure why, but not all the fields (this is the
			// same layout as the dacDialog above) get displayed.
			// But otherwise, things seem to work OK.
			//

			if (modelessDacDialog)
				(void) modelessDacDialog->doDialog();
			break;

		case cmModelessDialog:			// insert it as a modeless dialog

			if (modelessDacDialog)
				(void) modelessDacDialog->doDialog(1);
			modelessDlgPtr = modelessDacDialog->getDialog();
			break;

		case cmModelessRemove:			// remove it from the screen

			if (modelessDacDialog)
				modelessDacDialog->undoDialog(modelessDlgPtr);
			modelessDlgPtr = 0;
			break;

		case cmResponseTest:

			(void) respTest();
			break;

		case cmResponseBoxTest:

			(void) respBoxTest();
			break;

		case cmResponseBGITest:

			(void) respBGITest();
			break;

		case cmResponseTokenTest:

			(void) respTokenTest();
			break;

		case cmDelayTest:

			(void) delayTest(delayData, expDialog);
			break;

		default:

			he_error = ERR_COMMAND_UNIMPLEMENTED;
			break;
		}
		clearEvent(event);						// clear event after handling
	}
	if (he_error != ERR_NONE)					// temp; avoid error message
	{
		msgArea->errorHandler(he_error);
		he_error = ERR_NONE;
	}
}


/************************************************************************
** TFaceApp::idle
**
**		This routine provides a nice way to perform any background
** tasks. At present, the THeapView task is hardwired into place.
**
*************************************************************************/

void
TFaceApp::idle()								// background task-master
{
	TProgram::idle();

#if defined(TESTHEAP)									// { TESTHEAP

	heap->update();

#endif													// } TESTHEAP

}


/************************************************************************
** TFaceApp::face_dialogs()
**
**		Saves and restores all the dialog boxes; they amount to all
** the configuration we need.
**
**		I should learn to use streams on this stuff; later dude.
**
*************************************************************************/

void
TFaceApp::face_dialogs
(
	FileOperation rw,
	char *fname
)
{
	FILE *fptr;
	size_t items;				// number of items successfully written

	if (rw == FILE_WRITE)
	{
		if ((fptr = fopen(fname, "wb")) != NULL)
		{
			//items = fwrite(&subDialog, sizeof(subDialog), 1, fptr);
			items=0; //TEMPORARY
			fclose(fptr);
		}
		else
		{
			msgArea->errorHandler(ERR_FILE_OPEN);
		}
		if (items < 1)
			msgArea->errorHandler(ERR_FILE_WRITE);
	}
	else
	{
		if ((fptr = fopen(fname, "rb")) != NULL)
		{
			//items = fread(&subDialog, sizeof(subDialog), 1, fptr);
			fclose(fptr);
		}
		else
		{
			msgArea->errorHandler(ERR_FILE_OPEN);
		}
		if (items < 1)
			msgArea->errorHandler(ERR_FILE_READ);
	}
}


/************************************************************************
** TFaceApp::respTest()
**
**		Tests the response device system.
**
**		It is important to note that objects that were inserted into
** the Turbo Vision deskTop cannot just be deleted. Instead, they
** must be destroyed(), as shown below!
**
**		Very very important!!!
**
**		Note that, for the response feedback calls, you can press either
** 1 or 2, and the feedback lights will be on the same side if the
** response matched the parameter (either 1 or 2) of the call. Otherwise,
** they will be on the opposite side.
**
*************************************************************************/

int
TFaceApp::respTest ()
{
	int errcode = 0;

	ResponseDevice *r = new ResponseDevice
	(
		KEYBOARD_RESPONSE,
		TRect(2, 2, 70, 21),			// bounds of the window
		"Response Device",				// window's title
		1								// window number
	);
	if (r)								// always check it... saves grief
	{
		deskTop->insert(r);				// bring up the window

		msgArea->bannerHandler
		(
			"Press the 1 or 2 key at each pause until the window disappears"
		);
		r->startScreen();
		r->readyLight();
		r->pause();
		r->intervalLight(1);
		r->pause();
		r->intervalLight(2);
		r->pause();
		r->answerLight();
		r->pause();
		r->responseFeedback(1);			// try answering with left...
		r->pause();
		r->responseFeedback(1);			// then right... buttons
		r->pause();
		r->responseFeedback(2);			// try answering with left...
		r->pause();
		r->responseFeedback(2);			// then right... buttons
		r->pause();

		deskTop->destroy(r);			// with TV, just can't delete it!!!
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** TFaceApp::respBoxTest()
**
**		Tests the new ResponseBox device system.  Similar in concept
** to respTest().
**
*************************************************************************/

int
TFaceApp::respBoxTest ()
{
	int errcode = 0;

	ResponseBoxDevice *r = new ResponseBoxDevice
	(
		KEYBOARD_RESPONSE,
		TRect(2, 2, 70, 21),			// bounds of the window
		"Response Box Device",			// window's title
		1								// window number
	);
	if (r)								// always check it... saves grief
	{
		deskTop->insert(r);				// bring up the window

		msgArea->bannerHandler
		(
			"Press the 1 or 2 key at each pause until the window disappears"
		);
		r->startScreen();
		r->readyLight();
		r->pause();
		r->intervalLight(1);
		r->pause();
		r->intervalLight(2);
		r->pause();
		r->answerLight();
		r->pause();
		r->responseFeedback(1);			// try answering with left...
		r->responseFeedback(1);			// then right... buttons
		r->responseFeedback(2);			// try answering with left...
		r->responseFeedback(2);			// then right... buttons
		r->pause();

		deskTop->destroy(r);			// with TV, just can't delete it!!!
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** TFaceApp::respBGITest()
**
**		Tests the new ResponseBox device system.  Similar in concept
** to respTest().
**
*************************************************************************/

int
TFaceApp::respBGITest ()
{
	int errcode = 0;

	msgArea->bannerHandler
	(
		"Press the 1 or 2 key at each pause until the window disappears"
	);

	ScreenCombo screencombo;
	ResponseType rtype = KEYBOARD_RESPONSE;

	switch (expDialog.scrType)			// gotten from Experiment dialog box
	{
	case MAIN_SCREEN:

		screencombo = BGI_USE_MAIN_VGA;
		if (expDialog.responseDevice == MOUSE_RESPONSE)
			rtype = MOUSE_RESPONSE;
		break;

	case MONO_SCREEN:

		screencombo = BGI_USE_HERC;
		if (expDialog.responseDevice == MOUSE_RESPONSE)
			rtype = MOUSE_RESPONSE;
		break;

	case ALT_MONO_SCREEN:

		screencombo = BGI_USE_MONO_PATCH;
		if (expDialog.responseDevice == MOUSE_RESPONSE)
			rtype = ALT_MOUSE_RESPONSE;
		break;
	}

	TMouse::hide();						// { mouse cursor

	ResponseBGIDevice *r = new ResponseBGIDevice
	(
			rtype,						// KEYBOARD_RESPONSE, MOUSE_RESPONSE
			Rectangle(0, 10, 100, 80),	// bounds of the window in percent
			"Response Box BGI Device",	// window's title
			screencombo,				// use desired screen for graphics
			RF_STANDARD,				// the usual response-box style
			expDialog.afcIntervals		// number of intervals
	);
	if (r)								// always check it... saves grief
	{
		r->startScreen();
		r->readyLight();
		r->pause();
		r->intervalLight(1);
		r->pause();
		r->intervalLight(2);
		r->pause();
		r->answerLight();
		r->pause();
		r->responseFeedback(1);			// try answering with left...
		r->responseFeedback(1);			// then right... buttons
		r->responseFeedback(2);			// try answering with left...
		r->responseFeedback(2);			// then right... buttons
		r->pause();
		delete r;						// calls closegraph(), if applicable
		redraw();						// redraw application screen
	}
	else
		errcode = 1;

	TMouse::show();						// } mouse cursor

	return errcode;
}


/************************************************************************
** TFaceApp::respTokenTest()
*************************************************************************/

int
TFaceApp::respTokenTest ()
{
	int errcode = 0;

	msgArea->bannerHandler
	(
		"Press the 1 or 2 key at each pause until the window disappears"
	);

	ScreenCombo screencombo;
	ResponseType rtype = KEYBOARD_RESPONSE;

	switch (expDialog.scrType)			// gotten from Experiment dialog box
	{
	case MAIN_SCREEN:

		screencombo = BGI_USE_MAIN_VGA;
		if (expDialog.responseDevice == MOUSE_RESPONSE)
			rtype = MOUSE_RESPONSE;
		break;

	case MONO_SCREEN:

		screencombo = BGI_USE_HERC;
		if (expDialog.responseDevice == MOUSE_RESPONSE)
			rtype = MOUSE_RESPONSE;
		break;

	case ALT_MONO_SCREEN:

		screencombo = BGI_USE_MONO_PATCH;
		if (expDialog.responseDevice == MOUSE_RESPONSE)
			rtype = ALT_MOUSE_RESPONSE;
		break;
	}

	TMouse::hide();						// { mouse cursor

	ResponseBGITokenDevice *r = new ResponseBGITokenDevice
	(
		faceTokens,						// list of phonemes & box locations
		faceTokenCount,					// the number of them
		rtype,							// KEYBOARD_RESPONSE, MOUSE_RESPONSE
		Rectangle(0, 10, 100, 80),		// bounds in percent (unused)
		"Response Box BGI Device",		// window's title
		screencombo						// use desired screen for graphics
	);
	if (r)								// always check it... saves grief
	{
		r->startScreen();
		r->readyLight();
		r->pause();
		r->intervalLight(1);
		r->pause();
		r->intervalLight(2);
		r->pause();

		/****************************************************************
		** The following should first light the leftmost 4 lights on the
		** top row, then the leftmost 4 lights on the second row.
		*****************************************************************/

		r->responseFeedback(1);			// show token number 1 blinking
		r->responseFeedback(2);			// show token number 2 blinking
		r->responseFeedback(3);			// show token number 3 blinking
		r->responseFeedback(4);			// show token number 4 blinking
		r->responseFeedback(5);			// show token number 5 blinking
		r->responseFeedback(6);			// show token number 6 blinking
		r->responseFeedback(7);			// show token number 7 blinking
		r->responseFeedback(8);			// show token number 8 blinking
		r->pause();
		delete r;						// calls closegraph(), if applicable
		redraw();						// redraw application screen
	}
	else
		errcode = 1;

	TMouse::show();						// } mouse cursor

	return errcode;
}


/************************************************************************
** TFaceApp::delayTest()
**
**		Tests the response device system and the delay timer system.
** Similar to respTest(), but uses delays instead of pauses.
** The effect is to simulate the screen appearance (just once) of
** the 2AFC experimental procedure.
**
**		Later, we can add random-interval generation and looping,
** to be halted by pressing both mouse buttons.
**
**		I really don't like the way timers are handled here. I must
** rethink them.
**
*************************************************************************/

#define INTERVAL_MAX	2

#if !defined(USE_TV_FOR_DELAY_TEST)				// { USE_TV_FOR_DELAY_TEST

/************************************************************************
** This version uses the BGI graphics code.
*************************************************************************/

int
TFaceApp::delayTest
(
	DelayParameters &delays,
		ExpParameters &expdialog
)
{
	int errcode = 0;
	TimerControl *xTimers = 0;
	ResponseBGIDevice *xBox = 0;

	TMouse::hide();							// { mouse cursor

	xTimers = delayMaker(delays);			// delays
	if (xTimers == 0)
		errcode = 1;

	if (!errcode)
	{
		xBox = responseMaker(expdialog.scrType, expdialog.responseDevice);
		if (xBox == 0)
			errcode = 2;
	}

	if (!errcode)
	{
			TimerControl *before;			// DELAY_BEFORE_INTERVAL
			TimerControl *stim;				// DELAY_FOR_STIMULUS
			TimerControl *between;			// DELAY_BETWEEN_INTERVAL
			TimerControl *answer;			// DELAY_BEFORE_ANSWER
			TimerControl *feeddelay;		// FEEDBACK_DELAY
			TimerControl *feedback;			// FEEDBACK_DURATION
			TimerControl *extrafeed;		// EXTRA_FEEDBACK_DELAY

			before			= xTimers;
			stim			= before->next();
			between			= stim->next();
			answer			= between->next();
			feeddelay		= answer->next();
			feedback		= feeddelay->next();
			extrafeed		= feedback->next();

			/****************************************************************
			** As you can see, we use a linked list of timers.  Don't get
			** yourself thinking that we have an array of timers.
			*****************************************************************/

			int mkey;						// the response made by user

			xBox->startScreen();			// wait for user to get ready

			for (int trial = 0; trial < 100; trial++)
			{
				if (trial > 0)
					extrafeed->timerDelay();	// slow the pace for subject

				xBox->readyLight();				// turn on the Ready light
				(void) before->timerDelay();

				for (int interval = 1; interval <= 2; interval++)
				{
					xBox->intervalLight(interval);
					(void) stim->timerDelay();
					if (interval < INTERVAL_MAX)
						(void) between->timerDelay();
				}
				answer->timerDelay();			// delay before turning on
				xBox->answerLight();

				//
				// Hmmm, I can't see easy way to insert extra delay before
				// turning on feedback lights.
				//

				mkey = xBox->responseFeedback(1);

				if (mkey != BOTH_BUTTONS)
						(void) feedback->timerDelay();
				else
				{
						errcode = 3;
						break;
				}
			}
			// redraw() can't go here, because ~xBox() calls closegraph().
	}
	if (xBox)
	{
		delete xBox;
		// redraw();			// works OK here
	}
	if (xTimers)
		deleteTimer(xTimers);

	redraw();
	TMouse::show();							// } mouse cursor

	return errcode;
}

#else										// }{ USE_TV_FOR_DELAY_TEST


/************************************************************************
** This version uses the Turbo Vision windows code.
** I think it's broken.
*************************************************************************/

int
TFaceApp::delayTest
(
	DelayParameters *delays
)
{
	int errcode = 0;

	TimerControl *firsttimer = 0;
	TimerControl *timer = 0;
	TimerControl *previoustimer = 0;
	double dur;
	int goodone = 0;

	for (int i = 0; i < DELAY_MAX; i++)
	{
		dur = (double) delays->scaleFactor / 100.0;
		dur *= delays->timerDelay[i];

		timer = new TimerControl		// make another timer
		(
			deskTop,					// hook it to TV deskTop
			delays->baseID + i,			// give it an id number
			(unsigned) dur,				// pass the duration
			TIMER_IGNORE_COUNT
		);
		if (timer)
		{
			goodone++;
			if (i == 0)
				firsttimer = timer;
			else
				timer->attach(previoustimer);
			previoustimer = timer;
		}
		else
		{
			break;
		}
	}
	if (goodone < DELAY_MAX)			// some timers not get made?
	{
		deleteTimer(firsttimer);
		msgArea->bannerHandler("Timer creation error(s)");
		return errcode = 2;
	}

	TimerControl *before;				// DELAY_BEFORE_INTERVAL
	TimerControl *stim;					// DELAY_FOR_STIMULUS
	TimerControl *between;				// DELAY_BETWEEN_INTERVAL
	TimerControl *feedback;				// DELAY_FOR_FEEDBACK

	before		= firsttimer;
	stim		= before->next();
	between		= stim->next();
	feedback	= between->next();

	ResponseDevice *r = new ResponseDevice
	(
		expDialog.responseDevice,		// instead of KEYBOARD_RESPONSE,
		TRect(2, 2, 70, 21),			// bounds of the window
		"Response Device",				// window's title
		1								// window number
	);
	if (r)								// always check it... saves grief
	{
		deskTop->insert(r);				// bring up the window

		msgArea->bannerHandler
		(
			"Press the 1 or 2 key after the 4 lights blink"
		);

		/****************************************************************
		** As you can see, we use a linked list of timers.  Don't get
		** yourself thinking that we have an array of timers.
		*****************************************************************/

		int mkey;						// the response made by user

		r->startScreen();

		r->readyLight();
		errcode = before->timerDelay();

		for (int interval = 1; interval <= INTERVAL_MAX; interval++)
		{
			r->intervalLight(interval);
			errcode = stim->timerDelay();
			if (interval < INTERVAL_MAX)
				errcode = between->timerDelay();
		}
		r->answerLight();
		mkey = r->responseFeedback(1);

		if (mkey != BOTH_BUTTONS)
			errcode = feedback->timerDelay();

		deskTop->destroy(r);			// with TV, just can't delete it!!!

		if (mkey == BOTH_BUTTONS)
			msgArea->bannerHandler("You aborted! No problem...");
	}
	else
		errcode = 1;

	deleteTimer(firsttimer);

	return errcode;
}

#endif											// } USE_TV_FOR_DELAY_TEST


/************************************************************************
** TFaceApp::delayMaker()
**
**		Sets up the delay timer system, hooking it into the Turbo Vision
** desk-top.
**
*************************************************************************/

#define INTERVAL_MAX	2

TimerControl *
TFaceApp::delayMaker
(
	DelayParameters &delays			// Delays dialog box
)
{
	TimerControl *firsttimer = 0;
	TimerControl *timer = 0;
	TimerControl *previoustimer = 0;
	double dur;
	int goodone = 0;

	for (int i = 0; i < DELAY_MAX; i++)
	{
		dur = (double) delays.scaleFactor / 100.0;
		dur *= delays.timerDelay[i];

		timer = new TimerControl			// make another timer
		(
			deskTop,						// hook it to TV deskTop
			delays.baseID + i,				// give it an id number
			(unsigned) dur,					// pass the duration
			TIMER_IGNORE_COUNT
		);
		if (timer)
		{
			goodone++;
			if (i == 0)
					firsttimer = timer;
			else
					timer->attach(previoustimer);
			previoustimer = timer;
		}
		else
		{
			break;
		}
	}
	if (goodone < DELAY_MAX)				// some timers not get made?
	{
		deleteTimer(firsttimer);
		firsttimer = 0;
	}
	return firsttimer;
}


/************************************************************************
** responseMaker()
**
**		Uses dialog-box parameters to bring up a ResponseBoxDevice.
** Right now, can only handle a ResponseBGIDevice. Maybe later we'll
** figure out how to reliably bring up a Turbo Vision window response
** box (as implemented in RESPWIND.CPP in the FACE_TV library).
**
*************************************************************************/

ResponseBGIDevice *
TFaceApp::responseMaker
(
	ScreenSetup scrsetup,
	ResponseType responsetype
)
{
	ScreenCombo screencombo;
	ResponseType button = responsetype;

	switch (scrsetup)	
	{
	case ALT_MONO_SCREEN:						// for normal experimentation

		screencombo		= BGI_USE_MONO_PATCH;	// lab Herc screen & main VGA
		break;

	case MAIN_SCREEN:							// for checking out problems

		screencombo		= BGI_USE_MAIN_VGA;		// control-booth screen
		break;

	case MONO_SCREEN:							// in case mouse died

		screencombo		= BGI_USE_HERC;			// control-booth screen
		break;

		default:								// in case you're confused

		screencombo		= BGI_AUTO_DETECT;		// whatever you find
		break;
	}

	//
	// Here, we could perhaps create other kinds of devices. For
	// example, it would be nice to allow use of a TV window for
	// a response box. But the FACE_TV module RESPWIND still has some
	// minor problem somewhere.
	//

	ResponseBGIDevice *r = new ResponseBGIDevice
	(
		button,							// device for accepting responses
		Rectangle(0, 10, 100, 90),		// bounds of the window in percent
		"2AFC Response Box",			// window's title
		screencombo						// use desired screen for graphics
	);
		return r;
}


/************************************************************************
** Sample control-function for FACE_TV
**
**		All this function does is display the value on the desktop
** in a little dialog box.  As the mouse moves, both the original
** field and the value should scroll together
*************************************************************************/

static TDialog *showBox = (TDialog *) NULL;
static TDeskTop *faceDeskTop;			// indirect access to deskTop

int
showFloatInBox
(
	double value
)
{
	if (showBox)
	{
		closeBox(showBox, 0);
	}
	showBox = floatBox
	(
		faceDeskTop, 12, 10, "Ctrl", value
	);
	return 0;							// bogus error code for now
}


/************************************************************************
** FACE_TV main routine
**
**		Normally, we'd just declare "TFaceApp myApp;" and do a
** "myApp.run()", but we need to test if it was created, for error-
** checking.
**
*************************************************************************/


int
main()
{
	TFaceApp *myApp = new TFaceApp();

	if (myApp)
	{
		faceDeskTop = myApp->tBoxTop;	// need global access to desktop
		myApp->run();					// run the main program loop
		if (showBox)					// did we leave a showBox open?
			closeBox(showBox, 0);		// yes, get rid of it
		delete myApp;					// get rid of everything
		return 0;						// everything worked
	}
	else
	{
		return 1;						// could not run the program
	}
}

