/************************************************************************
**
** @(#)face_tv.h	11/14/94	Chris Ahlstrom
**
** C++
**
**	Contains, at present, an interface to the menu fields.
** This module is for the test program for the FACE_TV library.
**
**	Note that we use both old and new dialog-box code... the old
** stuff is represented by the TBox base class, and the new stuff is
** represented by the new member DialogBox *modelessDacDialog.
**
*************************************************************************/


#if !defined(FACE_TV_h)					// { FACE_TV_h
#define FACE_TV_h

#include "filedlg.h"		// FilePicker declarations
#include "heapview.h"		// for testing of the heap
#include "dlg_box.h"		// DialogBox support
#include "dosshell.h"		// DOS-shell support
#include "menubars.h"		// supports menu-bar application
#include "tbox.h"		// support dialog-box application
#include "screnmsg.h"		// supports menu-bar application
#include "tv_confg.h"		// SimpleConfiguration class
#include "x_delay.ph"		// DelayParameters class
#include "timerctl.h"		// controlling timers (TimerControl class)
#include "rbox_dev.h"		// ResponseBoxDevice class
#include "rbox_dbg.h"		// ResponseBGIDevice class
#include "rbox_dtk.h"		// ResponseBGITokenDevice class
#include "face_box.h"		// ExpParameters & ScreenSetup

extern int showFloatInBox
(
    double value
);


/************************************************************************
** TFaceApp
**
**	Obtains the menu/status bar initialization routines, the
** doDialog(), and shell() from the base objects.
** Note that TBox and DosShell could be considered "has-a" classes, and
** could be implemented as objects that are part of the TFaceApp class,
** instead of letting TFaceApp inherit from them.
**
**	But DosShell does have some private/protected members, so we'd
** need to use inheritance to get at its private members.  Anyway,
** I set it up this way before really knowing much about the concept
** of "has-a".
**
*************************************************************************/

class TFaceApp : public MenuBarApp, public TBox, private DosShell
{

public:

    TFaceApp();
    ~TFaceApp();

    virtual void handleEvent		// TV override
    (
	TEvent& event
    );
    void face_dialogs			// file dialogs
    (
	FileOperation rw,
	char *fname
    );
    virtual void idle();		// Updates heap view

    int respTest();			// ResponseDevice test
    int respBoxTest();			// ResponseBoxDevice test
    int respBGITest();			// ResponseBGIDevice test
    int respTokenTest();			// ResponseBGITokenDevice test
    int delayTest
    (
	DelayParameters &delays,
	ExpParameters &expdialog
    );
    TimerControl *delayMaker
    (
	DelayParameters &delays		// Delays dialog box
    );
    ResponseBGIDevice *responseMaker
    (
	ScreenSetup scrsetup,
	ResponseType responsetype
    );
    int modalTest();
    int modelessTest();
    int modelessRemove();

    UserMessages *msgArea;		// points to this class's errors
    ErrorCode appError;			// storage for the error code
    FilePicker *configFile;		// name of the configuration file
    TDialog *showBox;			// sample box for showing

protected:

    SimpleConfiguration *equipmentConfig;	// hard-wired list of equipment
    DialogBox *modelessDacDialog;		// for modeless dialog test
    TDialog *modelessDlgPtr;			// ditto

private:

    TWindowScreen *testsWindow;
    void initTFaceApp();		// initial set-up
    int textMode;			// current (desired) text mode
    THeapView *heap;			// Heap view for debugging

};


#endif							// } FACE_TV_h

