/************************************************************************
**
** @(#)randlist.h		10/17/94		Chris Ahlstrom
**
** Borland C++ Template Code
**
** Edit with tab-stops at 4.
**
** ------------------------
** 73340.26@compuserve.com
** ------------------------
**
**		Uses templates to create a family of functions that can
** randomize lists of items.
**
**		The list is randomized by using random_integers() to create
** a list of random numbers from 0 to number-1.  Call this list r.  Call
** the original presentationList p.  Call the randomized list q.  There
** are two ways to generate q:
**
**		1.	Make q have all the elements in p, but in the order specified
**			by r:
**
**				q[i] = p[r[i]]		for i = 0 to presentationCount-1
**
**			But this means we'd a copy of p from which to get the
**			original number, since it might have been overwritten
**			in an earlier i.
**
**			The routine scramble() implements this method.
**
**		2.	Swap the elements in p based on r:
**
**				s = q[i]			(save original value)
**				q[i] = p[r[i]]		(put one in from random location)
**				q[r[i]] = s			(save original value in the random
**									 location)
**
**			I had believed this is just as random, but I had no proof.
**			So I wrote randtest.c in the FACE library (which you might
**			not have access to), and found that this method makes a
**			number less probable in slots more distant from it.  For
**			example, in a list of 10 slots number from 0 to 9, slot
**			0 tends to most of the time to have 0's in it, and slot 9
**			tends to have 9's in it, while slot 9 never has a zero,
**			and slot 0 never has a nine.
**
**			semiScramble() implements this method.
**
*************************************************************************/

#if !defined(RANDLIST_h)								// { RANDLIST_h
#define RANDLIST_h


extern "C"
{
	#include "math_aux.h"	// random_integers (plain C function)
};



/************************************************************************
** semiScramble()
**
**		This function takes a list of items, and uses a list of random
** integers to help partly scramble the list, as per method 2 above.
**
**		Note the overloaded version below.
**
*************************************************************************/

template <class TTYPE> int
semiScramble
(
	TTYPE *items,
	int count
)
{
	int errcode = 0;
	int *rand_list = random_integers(count);				// plain C function

	if (rand_list)
	{
		errcode = semiScramble(rand_list, items, count);	// take advantage
		free(rand_list);
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** semiScramble() overload
**
**		This function takes a list of items, and uses a list of random
** integers to help partly scramble the list, as per method 2 above.
**
**		Same as the function above, but the list of random integers 
** is obtained externally (for situations in which the same randomization
** is needed for more than one list.
**
*************************************************************************/

template <class TTYPE> int
semiScramble
(
	int *rand_list,
	TTYPE *items,
	int count
)
{
	int errcode = 0;

	if (rand_list)
	{
		for (int i = 0; i < count; i++)
		{
			int swap = rand_list[i];

			if (swap < count)
			{
				TTYPE temp = items[i];

				items[i] = items[swap];
				items[swap] = temp;
			}
		}
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** scramble()
**
**		This function takes a list of items, copies them, then uses the
** copy to wholly scramble the list, as per method 1 above.
**
**		Note the overloaded version below.
**
*************************************************************************/

template <class TTYPE> int
scramble
(
	TTYPE *items,
	int count
)
{
	int errcode = 0;
	int *rand_list = random_integers(count);			// plain C function

	if (rand_list)
	{
		errcode = scramble(rand_list, items, count);	// take advantage
		free(rand_list);
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** scramble() overload
**
**		Same as the function above, but the list of random integers 
** is obtained externally (for situations in which the same randomization
** is needed for more than one list.
**
*************************************************************************/

template <class TTYPE> int
scramble
(
	int *rand_list,
	TTYPE *items,
	int count
)
{
	int errcode = 0;

	if (rand_list)
	{
		TTYPE *copyoflist = new TTYPE[count];

		if (copyoflist)
		{
			for (int ci = 0; ci < count; ci++)
				copyoflist[ci] = items[ci];

			for (int si = 0; si < count; si++)
				items[si] = copyoflist[rand_list[si]];

			delete copyoflist;
		}
		else
			errcode = 2;
	}
	else
		errcode = 1;

	return errcode;
}


#endif													// } RANDLIST_h
