/************************************************************************
**
** @(#)randtest.cpp			10/17/94	Chris Ahlstrom
**
**		Tests random_integers() in math_aux.c/h and the scramble() and
** semiScramble templates in randlist.h.
**
**		This program is essentially the same as the randtest.c file
** in the FACE library, but implemented with C++ techniques.  Note,
** though, that I haven't thrown out or reimplemented the function
** random_integers() in math_aux.c/h [which appears in both the FACE
** and FACE_TV libraries].
**
*************************************************************************/

#include <dir.h>		/* fnsplit()									*/
#include <stdio.h>		/* printf()										*/
#include <stdlib.h>		/* atoi()										*/
#include <string.h>		/* strcmp()										*/

#include "randlist.h"	/* scramble() and semiScramble() templates		*/


#define RANDTEST_VERSION "1.02"

#define ONE_LINER		((80-5)/4)


/************************************************************************
** Big Help
*************************************************************************/

#define RANDTEST_HELP													\
"RANDTEST v. " RANDTEST_VERSION "\n\n"									\
"Usage:  RANDTEST [number] [count]\n"									\
"Usage:  RANDLIST [number] [count]\n"									\
"\n"																	\
"	Generates \"number\" random numbers [default is 10] from 0 to\n"	\
"number-1, in random order.  These numbers are then shown.  If\n"		\
"less than 19 are shown, they can be shown on one line, for easy\n"		\
"verification of the \"randomness\".\n"									\
"	If count is given, then the list is generated count times.\n"		\
"	RANDTEST shows a generated list, while RANDLIST shows a swap-\n"	\
"generated list... which we hope is just as random.  RANDLIST is the\n"	\
"more thorough test.\n"


/************************************************************************
** Declarations
*************************************************************************/

extern int randtest1
(
	int number
);
extern int randtest2
(
	int number
);

static char rtDrive[MAXDRIVE];
static char rtDir[MAXDIR];
static char rtName[MAXFILE];
static char rtExt[MAXEXT];

static int origCount[ONE_LINER][ONE_LINER];		/* population matrix	*/
static int swapCount[ONE_LINER][ONE_LINER];		/* population matrix	*/

static int repCount;

/************************************************************************
** main()
*************************************************************************/

int
main
(
	int argc,
	char *argv[]
)
{
	int err = 0;
	char *numberstring = "10";
	int number;
	char *countstring = "1";
	int count;
	int useswapcode;

	(void) fnsplit(argv[0], rtDrive, rtDir, rtName, rtExt);

	if (stricmp(rtName, "randlist") == 0)
	{
		useswapcode = 1;
		printf("Will show two versions.\n");
	}
	else
	{
		useswapcode = 0;
	}

	if (argc >= 2)
	{
		if ((argv[1][0] <= '0') || (argv[1][0] >= '9'))
		{
			printf("%s", RANDTEST_HELP);
			exit(err);
		}
		else
		{
			numberstring = argv[1];
		}
		if (argc >= 3)
		{
			countstring = argv[2];
		}
	}

	number = atoi(numberstring);
	repCount = count = atoi(countstring);

	if (number <= 1)
	{
		err = 1;
		printf
		(
			"random_integers() can't handle lists shorter than 2 numbers.\n"
		);
	}
	else
	{
		int rc;

		if (useswapcode)
		{
			if (number <= ONE_LINER)
			{
				int pmi, pmj;

				for (pmi = 0; pmi < ONE_LINER; pmi++)
					for (pmj = 0; pmj < ONE_LINER; pmj++)
					{
						origCount[pmi][pmj] = swapCount[pmi][pmj] = 0;
					}
			}
			for (rc = 0; rc < count; rc++)
			{
				err = randtest2(number);
				if (err)
				{
					printf("%%%%\tRANDLIST: found some problems.\n");
					break;
				}
			}
			if (number <= ONE_LINER)
			{
				int pi, si;

				/********************************************************
				** Histogram for original random list.
				*********************************************************/

				printf("\n          ORIGINAL RANDOM LIST\n\n");
				printf("                Slot #\n\n");

				printf("    ");
				for (si = 0; si < number; si++)
					printf("%2d  ", si);
				printf("\n");

				printf("    ");
				for (si = 0; si < number; si++)
					printf("----");
				printf("\n");

				for (pi = 0; pi < number; pi++)
				{
					printf("%2d: ", pi);
					for (si = 0; si < number; si++)
						printf("%3d ", origCount[pi][si]);
					printf("\n");
				}

				/********************************************************
				** Histogram for list derived by swapping.
				*********************************************************/

				printf("\n          SWAPPING RANDOM LIST\n\n");
				printf("                Slot #\n\n");

				printf("    ");
				for (si = 0; si < number; si++)
					printf("%2d  ", si);
				printf("\n");

				printf("    ");
				for (si = 0; si < number; si++)
					printf("----");
				printf("\n");

				for (pi = 0; pi < number; pi++)
				{
					printf("%2d: ", pi);
					for (si = 0; si < number; si++)
						printf("%3d ", swapCount[pi][si]);
					printf("\n");
				}

			}
		}
		else
		{
			for (rc = 0; rc < count; rc++)
			{
				err = randtest1(number);
				if (err)
				{
					printf("%%%%\tRANDTEST: found some problems.\n");
					break;
				}
			}
		}
	}
	return err;
}


/************************************************************************
** randtest1()
**
**		The list is randomized by using random_integers() to create
** a list of random numbers from 0 to number-1.  Call this list r.  Call
** the original presentationList p.  Call the randomized list q.  There
** are two ways to generate q:
**
**		1.	As done by scramble() in randlist.h.
**
**		2.	As done by semiScramble() in randlist.h.
**
*************************************************************************/

int
randtest1
(
	int number
)
{
	int errcode = 0;

	int *orig_list = random_integers(number);		// [ uses malloc()

	if (orig_list)
	{
		int i;

		for (i = 0; i < number; i++)
		{
			if (number <= ONE_LINER)
			{
				printf("%2d ", orig_list[i]);
			}
			else
			{
				printf("%2d\n", orig_list[i]);
			}
		}
		if (number <= ONE_LINER)
			printf("\n");

		free(orig_list);							// ] so must be free()'d
	}
	else
	{
		printf("?Couldn't allocate list of random integers\n");
		errcode = 1;
	}
	return errcode;
}


int
randtest2
(
	int number
)
{
	int errcode = 0;								// return-code
	int *orig_list = new int[number];				// for plain scramble()

	if (orig_list)
	{
		int *swap_list = new int[number];

		if (swap_list)
		{
			int ni, pi;

			for (ni = 0; ni < number; ni++)			// start with straight list
			{
				orig_list[ni] = swap_list[ni] = ni;
			}
			(void) scramble(orig_list, number);		// scramble one of them
			(void) semiScramble						// use it to scramble other
			(
				orig_list, swap_list, number
			);

			if (repCount < (2*ONE_LINER))			// show simple scramble
				printf("Orig ");
			for (pi = 0; pi < number; pi++)
			{
				if (number <= ONE_LINER)
				{
					int picked;

					picked = orig_list[pi];
					if (repCount < (2*ONE_LINER))
						printf("%2d ", picked);
					origCount[picked][pi]++;		// count the incident
				}
				else
				{
					printf("%2d\n", orig_list[pi]);
				}
			}
			if ((number <= ONE_LINER) && (repCount < (2*ONE_LINER)))
				printf("\n");

			if (repCount < (2*ONE_LINER))			// show bad (semi) scramble
				printf("Swap ");
			for (pi = 0; pi < number; pi++)
			{
				if (number <= ONE_LINER)
				{
					int picked;

					picked = swap_list[pi];
					if (repCount < (2*ONE_LINER))
						printf("%2d ", picked);
					swapCount[picked][pi]++;		// count the incident
				}
				else
				{
					printf("%2d\n", swap_list[pi]);
				}
			}
			if ((number <= ONE_LINER) && (repCount < (2*ONE_LINER)))
				printf("\n");

			delete [] swap_list;
		}
		delete [] orig_list;
	}
	else
	{
		printf("?Couldn't allocate list of random integers\n");
		errcode = 1;
	}

	return errcode;
}


