/************************************************************************
**
** @(#)rbox_bas.h		01/07/96		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
**	C++; edit this module with tab-stops of 4.
**
**		This module supports generic response-box devices.  It should
** supercede the ResponseDevice class, which was very restrictive.
** The modules to supercede are:
**
**		RBUTTONS
**		RESPWIND
**
**		The model for a ResponseBoxDeviceBase is pretty simple.  We have
** manipulator functions that call functions for the response-button
** and response-light-drawing classes that ResponseBoxDeviceBase
** uses in a "has-a" relationship.
**
*************************************************************************/

#if !defined(RBOX_BAS_h)								// { RBOX_BAS_h
#define RBOX_BAS_h

#include "points.h"			// Rectangle class
#include "rbuttons.h"		// ResponseButton class
#include "rbox_int.h"		// ResponseBoxInterior class
#include "scrcombo.h"		// ScreenCombo & ResponseFeature enumerations


/************************************************************************
** ResponseBoxDeviceBase
**
**		We override the version of startScreen, in
** in order to provide the extra function of waiting for the user.
**
**		We add a responseFeedback() routine to wait for a key press
** and show the feedback lights.
**
*************************************************************************/

class ResponseBoxDeviceBase
{

public:

	ResponseBoxDeviceBase
	(
		ResponseType device,
		const Rectangle &bounds,
		const char *atitle = 0,					// title is optional
		ScreenCombo combo = BGI_SUBJECT_IS_USER,// most diasabled
		ResponseFeature style = RF_STANDARD,	// response-box style
		int itemcount = -1						// no. of important items
	);

	virtual ~ResponseBoxDeviceBase();

	virtual ResponseBoxSet startScreen		// bring up initial screen
	(
		int proceed = 0						// if 1, don't need to wait
	);
	virtual void readyLight();				// Warning! Trial is starting!
	virtual void responseOff();
	virtual void intervalLight
	(
		unsigned interval
	);
	virtual void answerLight();
	virtual int responseFeedback
	(
		int stimulus
	);
	virtual void pause ();					// waits for a keystroke
	virtual int isKeyed ();					// true if a key is pressed
	virtual int keyAbort ();				// true if last key-->aborted
	virtual void failureLight ();

protected:

	Rectangle boxBounds;					// size occupied on display
	ResponseButton *buttons;				// handles subject's input
	ScreenCombo screenCombo;				// for storage
	ResponseFeature responseStyle;			// response-box style
	int itemCount;							// intervals needed

	char boxTitle[32];

public:

	ResponseBoxInteriorBase *subInterior;	// access to interior

};


#endif													// } RBOX_BAS_h
