/************************************************************************
**
** @(#)rbox_dev.cpp		10/17/94		Chris Ahlstrom
**
** C++
**
**		This module defines the base class ResponseBoxDevice, which
** should provide a common interface for *all* response-box devices that
** we'll need for psychoacoustical and speech experiments, but DOES
** NOT, at present.  See rbox_dbg.cpp -- we need a base-class-in-common
** for these classes, but have'nt worked it out yet.
**
**		The class works by creating a response-window object for
** presenting visual feedback, and a button object for accepting
** responses from the user.
**
**		This module supercedes the RESPWIND module.  However, we've
** kept that module around, and it is still used in the test,
** even thought the old code makes the library larger.
**
*************************************************************************/

#define RBOX_DEV_cpp

#include <stdio.h>
#include <conio.h>
#include <ctype.h>

#include "mkbutton.h"			// ResponseButton creation class
#include "rbuttons.h"			// ResponseButton class

#include "rbox_dev.h"			// ResponseBoxDevice class
#include "rbox_tvi.h"			// ResponseTVInterior class


#define cpBlueWindow "\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F"
#define cpRespPalette cpBlueWindow	\
	RESPONSE_PAL INTERVAL_PAL READY_PAL ANSWER_PAL BACK_PAL


/************************************************************************
** ResponseBoxDevice
**
**		Can't make copies and such until we write the copy constructor
** and assignment operator.
**
*************************************************************************/

ResponseBoxDevice::ResponseBoxDevice
(
	ResponseType device,
	const TRect& bounds,
	const char *atitle,
	short anumber
) :
	TWindow		(bounds, atitle, anumber),
	TWindowInit	(ResponseBoxDevice::initFrame),
	buttons		(makeResponseButton(device)),
	subInterior	(0)
{
	TRect wbounds = getExtent();				// size of window
	wbounds.grow(-1, -1);						// fit it in window

	//
	// Here, we could select another type of ResponseBoxInterior
	// object.
	//

	ResponseTVInterior *interior;

	interior = new ResponseTVInterior		// make an interior
	(
		wbounds
	);
	if (interior)
	{
		insert(&interior->tvInterior);		// put its window in window
		subInterior = interior;
	}
	//if (subInterior)
	//subInterior->startState();			// NOW turn on the lights
}


/************************************************************************
** ~ResponseBoxDevice
*************************************************************************/

ResponseBoxDevice::~ResponseBoxDevice()
{
	//
	// In Turbo Vision, the interior of the window (pointed to by
	// subInterior), will automatically be deleted.  I doubt this
	// will hold true using Object Windows... check.
	//

	if (buttons)
		delete buttons;
}


/************************************************************************
** getPalette
*************************************************************************/

TPalette &
ResponseBoxDevice::getPalette() const
{
	static TPalette palette(cpRespPalette, sizeof(cpRespPalette)-1);
	return palette;
}


/************************************************************************
** startScreen
*************************************************************************/

ResponseBoxSet
ResponseBoxDevice::startScreen
(
	int proceed
)
{
	ResponseBoxSet status = BOX_RESPONSE_NORMAL;

	(void) subInterior->startState();			// first display
	if (!proceed)								// do we need to wait?
	{
		if										// wait for subject
		(
			buttons && (buttons->waitKey() == BOTH_BUTTONS)
		)
			status = BOX_RESPONSE_QUIT;
	}
	(void) subInterior->responseOff();			// turn off all lights

	return status;
}


/************************************************************************
** Encapsulating routines
*************************************************************************/

void
ResponseBoxDevice::readyLight ()
{
	(void) subInterior->readyLight();
}


void
ResponseBoxDevice::responseOff ()
{
	(void) subInterior->responseOff();
}


void
ResponseBoxDevice::intervalLight
(
	ushort interval
)
{
	(void) subInterior->intervalLight((int) interval);
}


void
ResponseBoxDevice::answerLight ()
{
	(void) subInterior->answerLight();
}


int
ResponseBoxDevice::responseFeedback
(
	int stimulus
)
{
	int tkey;

	(void) subInterior->answerLight();			// light answer light
	if (buttons)
		tkey = buttons->waitKey();				// wait for subject
	(void) subInterior->responseOff();			// turn off answer light

	if (tkey != BOTH_BUTTONS)					// exit button(s)?
	{
		if (tkey == BUTTON_3)					// 3-button mouse or keyboard
			tkey = 3;							// re-map it

		(void) subInterior->feedbackLight		// light feedback light
		(
			stimulus, tkey						// each is 1 to 3
		);
	}
	return tkey;
}


void
ResponseBoxDevice::pause ()
{
	if (buttons)
		(void) buttons->waitKey();				// wait for subject
}


int
ResponseBoxDevice::isKeyed ()
{
	int keyed = 0;

	if (buttons)
		keyed = buttons->readKey();

	return (keyed != 0);
}


int
ResponseBoxDevice::keyAbort ()
{
	int key = BOTH_BUTTONS;

	if (buttons)
		key = buttons->getLastResponse();

	return (key == BOTH_BUTTONS);
}

