//***********************************************************************
//
// @(#)rbox_nul.cpp		01/07/96		Chris Ahlstrom
//
// C++; edit with tab-stops at 4.
//
//		Provides the interface for ResponseBoxDevice (rbox_dbg modules)
// and ResponseBGIDevice (rbox_bgi modules).
//
//		All ResponseBoxNull descendants will have the same
// type of button devices available to them, but different types of
// screen representations of response lights and text.
//
//***********************************************************************

#define rbox_nul_cpp

#include "mkbutton.h"			// ResponseButton creation routine
#include "rbox_nul.h"			// ResponseBoxNull
#include "rbox_in0.h"			// ResponseBoxInteriorNull


//***********************************************************************
// ResponseBoxNull
//***********************************************************************

ResponseBoxNull::ResponseBoxNull
(
	const Rectangle &bounds,	// size occupied on display
	const char *atitle			// later, could write at top of screen
) :
	ResponseBoxDeviceBase(KEYBOARD_RESPONSE, bounds, atitle)
{
	subInterior = new ResponseBoxInteriorNull();
}


//***********************************************************************
// ~ResponseBoxNull
//***********************************************************************

ResponseBoxNull::~ResponseBoxNull()
{
}


//***********************************************************************
// startScreen
//
//		This version takes away control and viewing from the user.
// The experimenter will be viewing that statuses and making the
// responses.
//
//		However, we make this function similar to the base class, so
// that we can eventually post a little message or nice picture for the
// subject to view.
//
//		But, we don't take any steps if the subInterior can't be built.
//
//***********************************************************************

ResponseBoxSet
ResponseBoxNull::startScreen
(
	int proceed
)
{
	ResponseBoxSet status = BOX_RESPONSE_NORMAL;

	if (!buttons)
		status = BOX_RESPONSE_QUIT;
	else if (subInterior != 0)
	{
		(void) subInterior->startState();			// first display
		if (!proceed)								// do we need to wait?
		{
			if (buttons->waitKey() == BOTH_BUTTONS)	// wait for subject
				status = BOX_RESPONSE_QUIT;
		}
		(void) subInterior->responseOff();			// turn off all lights
	}
	return status;
}


//***********************************************************************
// Encapsulating routines
//
//		Only a few routines retain their base functionality.
// The rest are disabled [in the header file].
//
//***********************************************************************


int
ResponseBoxNull::responseFeedback
(
	int // stimulus
)
{
	int tkey;

	if (!buttons)
		tkey = BOTH_BUTTONS;
	else
	{
		answerLight();							// light answer light
		if (buttons)
			tkey = buttons->waitKey();			// wait for subject
		responseOff();							// turn off answer light

		if (tkey != BOTH_BUTTONS)				// exit button(s)?
		{
			if (tkey == BUTTON_3)				// 3-button mouse or keyboard
				tkey = 3;						// re-map it
		}
	}
	return tkey;
}


