/************************************************************************
**
** @(#)rbox_tvi.cpp		06/24/94		Chris Ahlstrom
**
** C++ version, requires Turbo Vision by Borland
**
**		This module defines the ResponseTVInterior class, which
** objectifies the abstract class ResponseBoxInterior for use with
** TurboVision windows.
**
**		ResponseTVInterior acts like a handle for the true interior
** drawing class, ResponseTView tvInterior.
**
*************************************************************************/

#define RBOX_TVI_cpp

#include "rbox_tvi.h"			// ResponseTVInterior class

#define TV_PALETTES
#include "rbox_afc.h"			// static virtual response box parameters


/************************************************************************
** ResponseTVInterior constructor and destructor.
**
**		Note that, since we allocate a semi-permanent textBox, it
** might prove wise to add an assignment operator and copy constructor.
**
**		resplightParameters[] and resplightCount[] are defined in
** respvga.h -- they implement a somewhat virtual (flexible) scheme
** of specifying the response-interior lights.
**
*************************************************************************/

ResponseTVInterior::ResponseTVInterior
(
	TRect &bounds
) :
	ResponseBoxInterior	(),
	tvInterior									// set up the TView
	(
		resplightParameters,
		resplightCount,
		bounds
	)
{
	lightCount = resplightCount;				// max number lights in class
	bitMaskList = new unsigned[lightCount];		// get some clean bits
	if (bitMaskList)
	{
		/****************************************************************
		** Set up LIGHT_PARAMETERS offsets
		*****************************************************************/

		for (int i = 0; i < lightCount; i++)		// for all lights
			bitMaskList[i] = ((unsigned) i << 8);	// set up parameter offsets

		/****************************************************************
		** Set up rest of LIGHT bits.  The call is to the default
		** startState() in rbox_int.cpp/h.
		*****************************************************************/

		startState();							// add usual stuff to bits
	}
}


ResponseTVInterior::~ResponseTVInterior ()
{
	if (bitMaskList)
		delete [] bitMaskList;
}


/************************************************************************
** virtual int startState() not overridden
*************************************************************************/

/************************************************************************
** virtual int drawGraphic() = 0;
** virtual int drawText() = 0;
** virtual int drawSpecial() = 0;
**
**		These functions do nothing, except for drawGraphic(), which
** need only tell the TView called "tvInterior" that the light is
** to be drawn as on or off.
**
*************************************************************************/

int
ResponseTVInterior::drawGraphic
(
	unsigned lpboffset,
	unsigned onstate
) 
{
	if (onstate)
		tvInterior.lightOn((int) lpboffset);
	else
		tvInterior.lightOff((int) lpboffset);

	return 0;
}


int
ResponseTVInterior::drawText
(
	unsigned	/* lpboffset*/ ,
	unsigned	/* onstate	*/
) 
{
	return 0;
}


int
ResponseTVInterior::drawSpecial
(
	int 		/* lightnumber*/ ,
	unsigned	/* onstate	  */
)
{
	return 0;
}



/************************************************************************
** drawThis()
*************************************************************************/

int
ResponseTVInterior::drawThis ()
{
	int errcode = ResponseBoxInterior::drawThis();	// set all lights

	if (readyForUse)
		tvInterior.drawView();

	return errcode;
}
