/************************************************************************
**
** @(#)umessage.h	03/28/94	Chris Ahlstrom
**
**	C++ Borland/Microsoft
**
**	Contains structures and definitions to support the base class
** for all message pointers, UserMessage.
**
*************************************************************************/

#if !defined(UMESSAGE_h)				// { UMESSAGE_h
#define UMESSAGE_h


/************************************************************************
** class UserMessage
**
**	UserMessage does not have any strictly private members...
** instead, they are protected.  At present, there are no functions
** and members that derived objects should not have access to.
**
*************************************************************************/

class UserMessage
{

public:

    UserMessage				// constructor
    (
	const char * const * const msglist,	// list of messages
	int msgcount				// number of them provided
    );
    UserMessage();			// default constructor

    virtual ~UserMessage();		// destructor

public:					// public access functions

    void hookMessages			// hook in messages at later time
    (
	const char * const * const msglist,	// list of messages
	int msgcount				// number of them provided
    );
    const char *Message ();		// pointer to last message
    const char *errorMessage ();	// pointer to last error message
    int isError ()			// get cheap error status in-line
    {
	return (errorCode != 0);
    }
    void setErrorCode			// set errorCode and szMessage
    (
	int index
    );
    int getErrorCode ()		// get code (derived objects only)
    {
	return errorCode;
    }

protected:				// internal message message fields

    /********************************************************************
    ** Callers can call isError() to see if an error occurred.  Some
    ** functions also return error codes.  In either case, if an error
    ** occurs, the caller can get the message by calling
    **
    **		getError()
    **
    ** Internally, derived objects will do one of two things:
    **
    **	    1.	Generate an errorCode and set szMessage equal to
    **		getMessageEntry(errorCode).  This is the default
    **		behavior of the internal function setErrorCode().
    **
    **	    2.	Make up their own message on the fly, and override
    **		setErrorCode() to let the caller have access to this
    **		message through getError().
    **
    ** Note the funny use of const... I want szMessageList to be a
    ** variable that points to a list of pointers, but I want the list
    ** itself to be constant (each element will always point to the
    ** same string), and what each element points to will also be
    ** constant.  In other words, I want szMessageList to be a variable
    ** pointer to a constant list of pointers to constant strings.
    ** Beautifully complex!
    **
    *********************************************************************/

    const char *getMessageEntry	// pointer to msg in internal list
    (
	int index
    );
    const char *szMessage;		// last message

public:					// to allow "has-a layering"

    const char * const * szMessageList;	// if used, a list of messages
    int szMessageCount;			// the number of messages in list
    int errorCode;			// message to the caller

};



#endif							// } UMESSAGE_h

